# -*- coding: utf-8 -*-
import unittest
import json
import hashlib

import mock

from facepy.exceptions import OAuthError

from pyramid.security import Allow

from pyramid_facebook.tests import conf

TEST_USER = 123


def _get_signed_request(user_id=None):
    from pyramid_facebook.lib import encrypt_signed_request
    return encrypt_signed_request(
        conf['facebook.secret_key'],
        {
            'user_id': user_id if user_id else TEST_USER,
            'user': {
                'country': 'ca'
                }
            }
        )

def _get_request(signed_request=None):
    request = mock.MagicMock()
    signed_request = signed_request if signed_request else _get_signed_request()
    request.params = {'signed_request': signed_request}
    request.registry.settings = conf.copy()
    return request


class TestFacebookCreditsContext(unittest.TestCase):

    @mock.patch('pyramid_facebook.security.json')
    def test_order_details(self, m_json):
        from pyramid_facebook.security import FacebookCreditsContext
        ctx = FacebookCreditsContext(_get_request())
        ctx.facebook_data = data = mock.MagicMock()

        self.assertEqual(m_json.loads.return_value, ctx.order_details)
        m_json.loads.assert_called_once_with(data['credits']['order_details'])

    def test_earned_currency_data(self):
        from pyramid_facebook.security import FacebookCreditsContext
        ctx = FacebookCreditsContext(_get_request())
        ctx.facebook_data = {
            'credits': {
                'order_details':json.dumps({
                    'items': [
                        {'data': '{"modified": "earned_currency_data"}'}
                        ]
                    })
                }
            }
        self.assertEqual(
            'earned_currency_data',
            ctx.earned_currency_data
            )

    def test_earned_currency_data_invalid_json(self):
        from pyramid_facebook.security import FacebookCreditsContext
        ctx = FacebookCreditsContext(_get_request())
        # with Invalid json data:
        ctx.facebook_data = {
            'credits': {
                'order_details':json.dumps({
                    'items': [
                        {'data': '{INVALID JSON "}'}
                        ]
                    })
                }
            }
        self.assertEqual(
            None,
            ctx.earned_currency_data
            )

    def test_order_info(self):
        from pyramid_facebook.security import FacebookCreditsContext
        ctx = FacebookCreditsContext(_get_request())
        ctx.facebook_data = mock.MagicMock()

        self.assertEqual(ctx.facebook_data["credits"]["order_info"], ctx.order_info)

    def test_item(self):
        from pyramid_facebook.security import FacebookCreditsContext
        ctx = FacebookCreditsContext(_get_request())
        ctx.facebook_data = mock.MagicMock()
        ctx.facebook_data.__getitem__.return_value.__getitem__.return_value = json.dumps(
            {'items': [{'title': 'an item'}]}
            )

        self.assertEqual(
            {'title': 'an item'},
            ctx.item,
            )


class TestSignedRequestContext(unittest.TestCase):

    def test_init(self):
        from pyramid_facebook.security import(
            SignedRequestContext,
            ViewCanvas,
            Authenticate,
            FacebookUser,
            RegisteredUser,
            )

        request = _get_request()

        ctx = SignedRequestContext(request)

        self.assertEqual(
            [
                (Allow, FacebookUser, Authenticate),
                (Allow, RegisteredUser, ViewCanvas),
                ],
            ctx.__acl__
            )
        self.assertEqual(
            {u'user_id': 123, u'algorithm': u'HMAC-SHA256', u'user': {u'country': u'ca'}},
            ctx.facebook_data
            )
        self.assertEqual({u'country': u'ca'}, ctx.user)
        self.assertEqual(u'ca', ctx.user_country)

    def test_init_malformated_signed_request(self):
        from pyramid_facebook.security import SignedRequestContext
        request = _get_request(signed_request='malformated')

        ctx = SignedRequestContext(request)
        self.assertIsNone(ctx.unauthenticated_userid())

    def test_init_no_user_id(self):
        from pyramid_facebook.lib import encrypt_signed_request
        from pyramid_facebook.security import SignedRequestContext
        request = _get_request(
            encrypt_signed_request(
                conf['facebook.secret_key'],
                {'no_user_id': '-_-'})
            )

        ctx = SignedRequestContext(request)
        self.assertIsNone(ctx.unauthenticated_userid())

        self.assertEqual([], ctx.effective_principals())

    def test_user_id_attribute_error(self):
        from pyramid_facebook.security import SignedRequestContext
        request = mock.MagicMock()
        request.params.get.return_value = None

        context = SignedRequestContext(request) # to raise AttributeError

        self.assertIsNone(context.unauthenticated_userid())

    def test_extract_user_id__error(self):
        from pyramid_facebook.security import SignedRequestContext

        request = _get_request(_get_signed_request('not a numeric value'))

        ctx = SignedRequestContext(request)

        self.assertIsNone(ctx.unauthenticated_userid())

    def test_repr(self):
        from pyramid_facebook.security import SignedRequestContext

        ctx = SignedRequestContext(_get_request())

        self.assertEqual(
            "<SignedRequestContext facebook_data={u'user_id': 123, u'user': {u'country': u'ca'}, u'algorithm': u'HMAC-SHA256'}>",
            ctx.__repr__()
            )


class TestFacebookAuthenticationPolicy(unittest.TestCase):

    def test_unauthenticated_userid(self):
        from pyramid_facebook.security import FacebookAuthenticationPolicy
        request = mock.Mock()

        fap = FacebookAuthenticationPolicy()
        self.assertEqual(
            request.context.unauthenticated_userid(),
            fap.unauthenticated_userid(request)
            )

    def test_unauthenticated_userid_attribute_error(self):
        from pyramid_facebook.security import FacebookAuthenticationPolicy
        request = mock.Mock()
        request.context.unauthenticated_userid.side_effect = AttributeError('no!')

        fap = FacebookAuthenticationPolicy()
        self.assertIsNone(fap.unauthenticated_userid(request))

    def test_effective_principals(self):
        from pyramid_facebook.security import FacebookAuthenticationPolicy, RegisteredUser
        request = mock.Mock()
        request.context.unauthenticated_userid.return_value = 123
        request.context.effective_principals.return_value = ['user']
        fap = FacebookAuthenticationPolicy()
        self.assertEqual(
            ['system.Everyone', 'system.Authenticated', 123, RegisteredUser, 'user'],
            fap.effective_principals(request)
            )

    def test_effective_principals_attribute_error(self):
        from pyramid_facebook.security import FacebookAuthenticationPolicy
        request = mock.Mock()
        request.context.unauthenticated_userid.side_effect = AttributeError('no!')
        request.context.effective_principals.side_effect = AttributeError('no!')
        fap = FacebookAuthenticationPolicy()
        self.assertEqual(['system.Everyone'], fap.effective_principals(request))


    def test_remember(self):
        from pyramid_facebook.security import FacebookAuthenticationPolicy

        fap = FacebookAuthenticationPolicy()

        request = mock.MagicMock()
        principal = "principal"
        kwargs = {"a": "a_val", "b": "b_val"}

        # FacebookAuthenticationPolicy does not support a remember/forget
        # mecanism, user is authenticated on each request
        self.assertListEqual([], fap.remember(request, principal, **kwargs))


    def test_forget(self):
        from pyramid_facebook.security import FacebookAuthenticationPolicy

        fap = FacebookAuthenticationPolicy()

        request = mock.MagicMock()

        # FacebookAuthenticationPolicy does not support a remember/forget
        # mecanism, user is authenticated on each request
        self.assertListEqual([], fap.forget(request))


class TestAccessTokenContext(unittest.TestCase):

    def test_init(self):
        from pyramid_facebook.security import AccessTokenContext
        request = mock.Mock()
        atc = AccessTokenContext(request)

        self.assertEqual(atc.request, request)

    @mock.patch('pyramid_facebook.security.GraphAPI')
    def test_user_dict(self, m_graph_api):
        from pyramid_facebook.security import AccessTokenContext, FacebookUser
        m_graph_api.return_value.get.return_value = dict(id=12345)
        request = mock.Mock()
        atc = AccessTokenContext(request)

        result = atc.user_dict

        self.assertEqual(12345, atc.unauthenticated_userid())
        self.assertEqual([FacebookUser], atc.effective_principals())

        request.params.get.assert_called_once_with('access_token')
        m_graph_api.assert_called_once_with(request.params.get.return_value)
        m_graph_api.return_value.get.assert_called_once_with('me', retry=0)


    @mock.patch('pyramid_facebook.security.GraphAPI')
    def test_user_dict_facepy_error(self, m_graph_api):
        from pyramid_facebook.security import AccessTokenContext
        m_graph_api.return_value.get.side_effect = OAuthError(code=190, message='OAuth Error')
        request = mock.Mock()

        atc = AccessTokenContext(request)
        result = atc.user_dict

        self.assertEqual(dict(), result)
        self.assertIsNone(atc.unauthenticated_userid())
        self.assertEqual([], atc.effective_principals())


class TestAdminContext(unittest.TestCase):

    @mock.patch('pyramid_facebook.security.GraphAPI')
    def test_effective_principals(self, m_graph_api):
        from pyramid_facebook.security import AdminContext, UpdateSubscription
        m_graph_api.return_value.get.return_value = dict(data=[dict(id=12345), dict(id=54321)])
        request = mock.MagicMock()

        ac = AdminContext(request)

        result = ac.effective_principals()

        for i in range(2):
            self.assertEqual(
                mock.call(request.params.get.return_value),
                m_graph_api.call_args_list[i]
                )

        self.assertEqual(
            mock.call('me/accounts', retry=0),
            m_graph_api.return_value.get.call_args_list[1]
            )

        self.assertEqual(
            ((Allow, request.registry.settings.__getitem__.return_value, UpdateSubscription),),
            ac.__acl__
            )

        self.assertEqual([12345, 54321], result)


    @mock.patch('pyramid_facebook.security.GraphAPI')
    def test_effective_principals_facepy_error(self, m_graph_api):
        from pyramid_facebook.security import AdminContext
        m_graph_api.return_value.get.side_effect = OAuthError(code=190, message='OAuth Error')
        request = mock.MagicMock()

        ac = AdminContext(request)
        result = ac.effective_principals()

        self.assertEqual([], result)


class TestRealTimeNotificationContext(unittest.TestCase):

    def test_init(self):
        from pyramid_facebook.security import (
            RealTimeNotificationContext,
            Allow,
            XHubSigned,
            NotifyRealTimeChanges
            )

        request = mock.Mock()

        rtnc = RealTimeNotificationContext(request)

        self.assertEqual(request, rtnc.request)
        self.assertIsNone(rtnc.principals)
        self.assertEqual(
            ((Allow, XHubSigned, NotifyRealTimeChanges), ),
            rtnc.__acl__
            )

    @mock.patch('pyramid_facebook.security.hmac')
    def test_effective_principals(self, m_hmac):
        from pyramid_facebook.security import (
            RealTimeNotificationContext,
            XHubSigned,
            )

        request = mock.MagicMock()
        request.headers = {'X-Hub-Signature': 'sha1=signature'}

        m_hmac.new.return_value.hexdigest.return_value = 'signature'

        rtnc = RealTimeNotificationContext(request)
        result = rtnc.effective_principals()

        self.assertEqual((XHubSigned,), result)

        m_hmac.new.assert_called_once_with(
            request.registry.settings.__getitem__.return_value,
            request.body,
            hashlib.sha1
            )
        m_hmac.new.return_value.hexdigest.assert_called_once_with()

    @mock.patch('pyramid_facebook.security.hmac')
    def test_effective_principals_bad_signature(self, m_hmac):
        from pyramid_facebook.security import (
            RealTimeNotificationContext,
            )

        request = mock.MagicMock()
        request.headers = {'X-Hub-Signature': 'sha1=signature'}

        m_hmac.new.return_value.hexdigest.return_value = 'not the same signature'

        rtnc = RealTimeNotificationContext(request)
        result = rtnc.effective_principals()

        self.assertEqual(tuple(), result)



# -*- coding: utf-8 -*-
import logging
import functools

import venusian
from pyramid.view import view_config
from pyramid.httpexceptions import HTTPOk

from pyramid_facebook.lib import request_params_predicate
from pyramid_facebook.events import (
    DisputedOrder,
    EarnedCurrencyOrder,
    PlacedItemOrder,
    RefundedOrder,
    )
from pyramid_facebook.security import ViewCanvas

log = logging.getLogger(__name__)


def includeme(config):
    """Adds routes related to facebook credits.

    Routes Added:

    * ``facebook_payments_get_items`` associated to url ``/{namespace}/credits``.

    * ``facebook_payments_status_update_placed`` associated to url
      ``/{namespace}/credits``.

    * ``facebook_payments_status_update_disputed`` associated to url
      ``/{namespace}/credits``.

    * ``facebook_payments_status_update_refunded`` associated to url
      ``/{namespace}/credits``.
    """
    log.debug('Adding route "facebook_payments_get_items".')
    config.add_route(
        'facebook_payments_get_items',
        '/credits',
        request_method='POST',
        custom_predicates=[
            request_params_predicate(
                'signed_request',
                'buyer',
                'receiver',
                'order_id',
                'order_info',
                method='payments_get_items',
                )
            ],
        factory='pyramid_facebook.security.FacebookCreditsContext',
        )

    update_predicate = request_params_predicate(
        'signed_request',
        'order_details',
        'order_id',
        method='payments_status_update',
        )

    log.debug('Adding route "facebook_payments_status_update_placed".')
    config.add_route(
        'facebook_payments_status_update_placed',
        '/credits',
        request_method='POST',
        custom_predicates=[
            update_predicate,
            request_params_predicate(status='placed'),
            ],
        factory='pyramid_facebook.security.FacebookCreditsContext',
        )

    log.debug('Adding route "facebook_payments_status_update_disputed".')
    config.add_route(
        'facebook_payments_status_update_disputed',
        '/credits',
        request_method='POST',
        custom_predicates=[
            update_predicate,
            request_params_predicate(status='disputed'),
            ],
        factory='pyramid_facebook.security.FacebookCreditsContext',
        )

    log.debug('Adding route "facebook_payments_status_update_refunded".')
    config.add_route(
        'facebook_payments_status_update_refunded',
        '/credits',
        request_method='POST',
        custom_predicates=[
            update_predicate,
            request_params_predicate(status='refunded'),
            ],
        factory='pyramid_facebook.security.FacebookCreditsContext',
        )

    config.scan(package='pyramid_facebook.credits')


class facebook_payments_get_items(object):
    """Decorator to register the function to process facebook credits
    `payments_get_items <http://developers.facebook.com/docs/credits/callback/#payments_get_items>`_.

    Decorated function receives 2 positional parameters:

    * ``context``: The :class:`~pyramid_facebook.security.FacebookCreditsContext`
      the request is associated with. ``context.facebook_data["user"]`` gives
      information about user's locale which would permit to return different
      languages.

    * ``request``: The request itself.

    It is possible to access `order_info` via :attr:`context.order_info
    <pyramid_facebook.security.FacebookCreditsContext.order_info>`:

    Decorated function must return a dictionary structured as::

        {
            # Required:
            "title":       "100 diamonds",
            "description": "100 shiny diamonds!",
            "price":       1000,
            "image_url":   "http://hadriendavid.com/images/100dimonds.png",

            # Optional (according to facebook doc):
            "item_id": "123",
            "data":    "whatever"
        }

    Example::

        @facebook_payments_get_items()
        def get_item(context, request):
            ...
            return {
                "title": a_title,
                "description": a_description,
                "price": a_price_in_facebook_credits,
                "image_url": an_image_url
                }
    """
    def __call__(self, wrapped):
        # as documented on http://docs.pylonsproject.org/projects/pyramid/en/1.3-branch/narr/hooks.html#registering-configuration-decorators
        venusian.attach(wrapped, self._register)

        log.debug('facebook_payments_get_items provided by %s', wrapped)
        return wrapped

    def _register(self, scanner, name, wrapped):
        """Register view to pyramid framework.
        """
        config = scanner.config
        config.add_view(
            view=self._decorate(wrapped),
            permission=ViewCanvas,
            route_name='facebook_payments_get_items',
            renderer='json',
            )
        log.debug('Adding views %s for route facebook_payments_get_items', wrapped)

    def _validate(self, item_content):
        "Validate item content"
        if not isinstance(item_content, dict):
            raise TypeError('dict expected, received %s' % type(item_content))
        expected = {
            'title'      : (str, unicode,),
            'description': (str, unicode,),
            'price'      : (int,         ),
            'image_url'  : (str, unicode,),
            }
        for k, v in expected.iteritems():
            if type(item_content.get(k)) not in v:
                raise TypeError('Expected %r, received %r' % (
                    expected,
                    item_content
                    ))
        # title must not be empty
        if len(item_content['title']) < 1:
            raise ValueError('item title must not be an empty string')
        return item_content

    def _decorate(self, wrapped):
        @functools.wraps(wrapped)
        def wrapper(context, request):
            log.debug(
                'facebook_payments_get_items call with facebook_data=%s',
                context.facebook_data
                )
            result = self._validate(wrapped(context, request))
            return {
                "content": [result],
                "method": "payments_get_items"
                }
        return wrapper



@view_config(route_name='facebook_payments_status_update_disputed')
def facebook_payments_status_update_disputed(context, request):
    try:
        log.debug('Order Status Update Disputed')
        request.registry.notify(DisputedOrder(context, request))
    except Exception:
        log.exception(
            'facebook_payments_status_update_disputed with context=%r, '
            'request.params=%r',
            context,
            request,
            )
    return HTTPOk()


@view_config(route_name='facebook_payments_status_update_refunded')
def facebook_payments_status_update_refunded(context, request):
    try:
        log.debug('Order Status Update Refunded')
        request.registry.notify(RefundedOrder(context, request))
    except Exception:
        log.exception(
            'facebook_payments_status_update_refunded with context=%r, '
            'request.params=%r.',
            context,
            request,
            )
    return HTTPOk()


@view_config(
    route_name='facebook_payments_status_update_placed',
    permission=ViewCanvas,
    renderer='json'
    )
def facebook_payments_status_update_placed(context, request):
    if context.earned_currency_data:
        log.debug('Order Status Update - Earned App Currency')
        event_class = EarnedCurrencyOrder
    else:
        log.debug('Order Status Update - Placed Item Order')
        event_class = PlacedItemOrder
    try:
        request.registry.notify(event_class(context, request))
    except Exception:
        log.exception(
            'facebook_payments_status_update_placed with context=%r, '
            'request.params=%r.',
            context,
            request,
            )
        status = 'canceled'
    else:
        log.debug(
            'facebook_payments_status_update_placed settled %r',
            context.order_details,
            )
        status = 'settled'
    return {
        'content': {
            'status': status,
            'order_id': context.order_details['order_id'],
            },
        'method': 'payments_status_update',
        }



# -*- coding: utf-8 -*-
import hashlib
import hmac
import json
import logging

from facepy import FacepyError, GraphAPI, SignedRequest
from facepy.exceptions import SignedRequestError

from pyramid.authentication import CallbackAuthenticationPolicy
from pyramid.decorator import reify
from pyramid.security import Allow

log = logging.getLogger(__name__)

ViewCanvas = 'view_canvas'
Authenticate = 'authenticate'
UpdateSubscription = 'update_subscription'
NotifyRealTimeChanges = 'notify_real_time_changes'

FacebookUser = 'facebook-user'
RegisteredUser = 'registered-user'
AdminUser = 'admin_user'
XHubSigned = 'x-hub-signed' # real time updates https://developers.facebook.com/docs/reference/api/realtime/


class SignedRequestContext(object):
    "Security context for facebook signed request routes."

    __acl__ = [
        (Allow, FacebookUser, Authenticate),
        (Allow, RegisteredUser, ViewCanvas),
        ]

    def __init__(self, request):
        self.request = request

    def unauthenticated_userid(self):
        try:
            return int(self.facebook_data["user_id"])
        except KeyError:
            # user_id not in facebook_data => user has not authorized app
            log.debug('User has not authorized app.')
        except ValueError:
            log.warn('Invalid user id %r', self.facebook_data["user_id"])
        except TypeError:
            # in case signature is malformated. fb_data inexists
            # no need to log as it is already done in __init__
            pass
        return None

    def effective_principals(self):
        if self.unauthenticated_userid():
            return [FacebookUser]
        return []

    @reify
    def facebook_data(self):
        """Contains facebook data provided in ``signed_request`` parameter
        decrypted with :meth:`SignedRequest.parse <facepy.SignedRequest.parse>`.
        """
        if self.request.params.get('signed_request'):
            try:
                return SignedRequest.parse(
                    self.request.params['signed_request'],
                    self.request.registry.settings['facebook.secret_key'],
                    )
            except SignedRequestError as e:
                log.warn(
                    '%r with signature: %s',
                    e,
                    self.request.params['signed_request']
                    )
        return None

    @reify
    def user(self):
        return self.facebook_data['user']

    @reify
    def user_country(self):
        return self.user['country']

    def __repr__(self):
        return '<%s facebook_data=%r>' % (
            self.__class__.__name__,
            self.facebook_data
            )


class FacebookCreditsContext(SignedRequestContext):
    "Context for facebook credits callback requests."

    @reify
    def order_details(self):
        """Order details received in `facebook credits callback for payment
        status updates <http://developers.facebook.com/docs/credits/callback/#payments_status_update>`_."""
        return json.loads(
            self.facebook_data['credits']['order_details']
            )

    @reify
    def order_info(self):
        """Order info being the order information passed when the `FB.ui method
        <http://developers.facebook.com/docs/reference/javascript/FB.ui/>`_
        is invoked."""
        return self.facebook_data["credits"]["order_info"]

    @reify
    def earned_currency_data(self):
        """Modified field received in `facebook credits callback for payment
        status update for earned app currency
        <http://developers.facebook.com/docs/credits/callback/#payments_status_update_earn_app_currency>`_."""
        data = self.item['data']
        if data:
            try:
                data = json.loads(data)
                data = data['modified'] if 'modified' in data else None
            except:
                data = None
        return data

    @reify
    def item(self):
        """The item info as passed when `FB.ui method
        <http://developers.facebook.com/docs/reference/javascript/FB.ui/>`_
        is invoked."""
        return self.order_details['items'][0]


class AccessTokenContext(object):

    def __init__(self, request):
        self.request = request

    @reify
    def user_dict(self):
        self.access_token = self.request.params.get('access_token')
        if self.access_token:
            try:
                return GraphAPI(self.access_token).get('me', retry=0)
            except FacepyError as e:
                log.warn('Authentication failed: %r', e)
        return dict()

    def unauthenticated_userid(self):
        return long(self.user_dict['id']) if 'id' in self.user_dict else None

    def effective_principals(self):
        p = []
        if self.unauthenticated_userid():
            p.append(FacebookUser)
        return p


class AdminContext(AccessTokenContext):
    """Context which defines principals as facebook application id list that user
    administrates."""

    def __init__(self, request):
        super(AdminContext, self).__init__(request)
        self.principals = None
        self.__acl__ = (
            (Allow, request.registry.settings['facebook.app_id'], UpdateSubscription),
            )

    def effective_principals(self):
        if self.principals is None:
            self.principals = super(AdminContext, self).effective_principals()
            # check what apps user owns.
            try:
                accounts = GraphAPI(self.access_token).get('me/accounts', retry=0)
                self.principals.extend([a['id'] for a in accounts['data']])
            except FacepyError as e:
                log.warn('Get accounts failed: %r', e)
        return self.principals


class RealTimeNotificationContext(object):
    "Context for real-time changes notification route."

    __acl__ = (
        (Allow, XHubSigned, NotifyRealTimeChanges),
        )

    def __init__(self, request):
        self.request = request
        self.principals = None

    def effective_principals(self):
        if self.principals is None:
            # route predicates already check presence of X-Hub-Signature header
            sig = self.request.headers['X-Hub-Signature']
            verif = hmac.new(
                self.request.registry.settings['facebook.secret_key'],
                self.request.body,
                hashlib.sha1
                ).hexdigest()
            if sig == ('sha1=%s' % verif):
                self.principals = (XHubSigned, )
            else:
                log.warn(
                    'X-Hub-Signature invalid - expected %s, received %s',
                    verif,
                    sig,
                    )
                self.principals = tuple()
        return self.principals


class FacebookAuthenticationPolicy(CallbackAuthenticationPolicy):

    def unauthenticated_userid(self, request):
        try:
            return request.context.unauthenticated_userid()
        except AttributeError:
            return None

    def effective_principals(self, request):
        p = super(FacebookAuthenticationPolicy, self).effective_principals(request)
        try:
            p.extend(request.context.effective_principals())
        except AttributeError:
            pass
        return p

    def remember(self, request, principal, **kwargs):
        return []

    def forget(self, request):
        return []

    def callback(self, user_id, request):
        # by default, having a user id means that user is registered
        return [RegisteredUser]



# -*- coding: utf-8 -*-
import logging
import pprint

from pyramid.authorization import ACLAuthorizationPolicy

from pyramid_facebook.security import FacebookAuthenticationPolicy

log = logging.getLogger(__name__)


def includeme(config):
    """Executed when including ``pyramid_facebook``.

    ``pyramid_facebook`` setup:

    * ACL Authorization with :class:`~pyramid.authorization.ACLAuthorizationPolicy`
      using :meth:`config.set_authorization_policy <pyramid.config.Configurator.set_authorization_policy>`
    * Authentication with :class:`~pyramid_facebook.security.FacebookAuthenticationPolicy`
      using :meth:`config.set_authentication_policy <pyramid.config.Configurator.set_authentication_policy>`

    """
    settings = config.registry.settings
    log.debug('Configuration settings: %s', pprint.pformat(settings))

    config.set_authentication_policy(FacebookAuthenticationPolicy())
    config.set_authorization_policy(ACLAuthorizationPolicy())

    path = '/%s' % settings['facebook.namespace']

    config.include('pyramid_facebook.auth',      route_prefix=path)
    config.include('pyramid_facebook.canvas',    route_prefix=path)
    config.include('pyramid_facebook.credits',   route_prefix=path)
    config.include('pyramid_facebook.real_time', route_prefix=path)
    config.commit()
